<?php
/************************************************************************
 * MXChange v0.2.1                                    Start: 03/25/2004 *
 * ===============                              Last change: 09/29/2004 *
 *                                                                      *
 * -------------------------------------------------------------------- *
 * File              : extensions.php                                   *
 * -------------------------------------------------------------------- *
 * Short description : Extension management                             *
 * -------------------------------------------------------------------- *
 * Kurzbeschreibung  : Erweiterungen-Management                         *
 * -------------------------------------------------------------------- *
 *                                                                      *
 * -------------------------------------------------------------------- *
 * Copyright (c) 2003 - 2008 by Roland Haeder                           *
 * For more information visit: http://www.mxchange.org                  *
 *                                                                      *
 * This program is free software; you can redistribute it and/or modify *
 * it under the terms of the GNU General Public License as published by *
 * the Free Software Foundation; either version 2 of the License, or    *
 * (at your option) any later version.                                  *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,               *
 * MA  02110-1301  USA                                                  *
 ************************************************************************/

// Some security stuff...
if (!defined('__SECURITY')) {
	$INC = substr(dirname(__FILE__), 0, strpos(dirname(__FILE__), "/inc") + 4) . "/security.php";
	require($INC);
}

// Load the extension and maybe found language and function files.
function LOAD_EXTENSION ($ext_name, $EXT_LOAD_MODE = "", $EXT_VER = "", $dry_run = false, &$SQLs = array()) {
	global $EXT_LOADED, $_CONFIG, $CSS, $cacheMode, $EXT_VER_HISTORY;
	global $INC_POOL, $EXT_UPDATE_DEPENDS, $EXT_DEPRECATED, $UPDATE_NOTES;
	global $EXT_VERSION, $EXT_ALWAYS_ACTIVE;

	// Init array
	$INC_POOL = array();

	// Is the extension already loaded?
	//* DEBUG: */ DEBUG_LOG(__FUNCTION__, __LINE__, "Loading extension {$ext_name}, mode={$EXT_LOAD_MODE}, ver={$EXT_VER}.");
	if ((isset($EXT_LOADED['ext'][$ext_name])) && (empty($EXT_LOAD_MODE))) {
		// Debug message
		DEBUG_LOG(__FUNCTION__, __LINE__, sprintf("Extension %s already loaded.", $ext_name));
		return false;
	} // END - if

	// Construct FQFN for extension file
	$extInclude = sprintf("%sinc/extensions/ext-%s.php", PATH, $ext_name);

	// Is the extension file NOT there?
	if (!FILE_READABLE($extInclude)) {
		// Debug message
		DEBUG_LOG(__FUNCTION__, __LINE__, sprintf("Extension %s not found.", $ext_name));

		// Abort here
		return false;
	} // END - if

	// Construct FQFN for language file
	$langInclude = sprintf("%sinc/language/%s_%s.php", PATH, $ext_name, GET_LANGUAGE());

	// Is this include there?
	if ((FILE_READABLE($langInclude)) && (!isset($EXT_LOADED['lang'][$ext_name]))) {
		// Then load it
		//* DEBUG: */ DEBUG_LOG(__FUNCTION__, __LINE__, "--- Language loaded.");
		$EXT_LOADED['lang'][$ext_name] = true;
		require($langInclude);
	} // END - if

	// Construct FQFN for functions file
	$funcsInclude = sprintf("%sinc/libs/%s_functions.php", PATH, $ext_name);

	// Is this include there?
	if ((FILE_READABLE($funcsInclude)) && (!isset($EXT_LOADED['funcs'][$ext_name]))) {
		// Then load it
		//* DEBUG: */ DEBUG_LOG(__FUNCTION__, __LINE__, "--- Functions loaded.");
		$EXT_LOADED['funcs'][$ext_name] = true;
		require($funcsInclude);
	} // END - if

	// Extensions are not deprecated by default
	$EXT_DEPRECATED = "N";

	// Extensions are not always active by default
	$EXT_ALWAYS_ACTIVE = "N";

	// By default an extension update does not depend on other extensions
	$EXT_UPDATE_DEPENDS = "";

	// Extension update notes
	$UPDATE_NOTES = "";

	// Include the extension file
	//* DEBUG: */ DEBUG_LOG(__FUNCTION__, __LINE__, "Extension loaded.");
	require($extInclude);

	// Is this extension deprecated?
	if ($EXT_DEPRECATED == "Y") {
		// Deactivate the extension
		DEACTIVATE_EXTENSION($ext_name);

		// Abort here
		return false;
	} // END - if

	// Mark it as loaded in normal mode
	if (empty($EXT_LOAD_MODE)) {
		// Mark it now...
		$EXT_LOADED['ext'][$ext_name] = true;
	} // END - if

	// All fine!
	return true;
}

// Registeres an extension and possible update depencies
function EXTENSION_REGISTER ($ext_name, $task_id, $dry_run = false, $logout = true) {
	global $UPDATE_NOTES, $INC_POOL, $cacheInstance;
	global $EXT_VER_HISTORY, $NOTES, $EXT_ALWAYS_ACTIVE, $EXT_VERSION;
	global $EXT_UPDATE_DEPENDS, $SQLs;

	// This shall never do a non-admin user!
	if (!IS_ADMIN()) return false;

	// Is this extension already installed?
	if (EXT_IS_ACTIVE($ext_name)) return false;

	// Init variables
	$ret = false; $SQLs = array();
	$NOTES = "";
	$INC_POOL = array();

	// By default the language prefix is the extension's name
	// @TODO: Do we really need this one anymore? Can't we just take $ext_name and done?
	// By default we have no failtures
	$EXT_REPORTS_FAILURE = false;

	// Does this extension exists?
	if (LOAD_EXTENSION($ext_name, "register", "", $dry_run, $SQLs)) {
		// And run possible updates
		$history = $EXT_VER_HISTORY;
		foreach ($history as $ver) {
			//* DEBUG: */ DEBUG_LOG(__FUNCTION__, __LINE__, "ext_name={$ext_name}, ext_ver={$ver}");
			// Load extension in update mode
			LOAD_EXTENSION($ext_name, "update", $ver, $dry_run, $SQLs);

			// Do we have an update?
			if (((GET_EXT_VERSION("sql_patches") != "") && (getConfig('verbose_sql') == "Y")) || (!EXT_IS_ACTIVE("sql_patches"))) {
				if (!empty($UPDATE_NOTES)) {
					// Update notes found
					$NOTES .= ("<div class=\"update_notes\"><strong>v".$ver.":</strong><br />".$UPDATE_NOTES."</div>");
					$UPDATE_NOTES = "";
				} elseif (($ver == "0.0") || ($ver == "0.0.0")) {
					// Initial release
					$NOTES .= ("<div class=\"update_notes\"><strong>v".$ver.":</strong><br />".INITIAL_RELEASE."</div>");
				} else {
					// No update notes found!
					$NOTES .= ("<div class=\"update_notes\"><strong>v".$ver.":</strong><br /><I>".NO_UPDATE_NOTES."</I></div>");
				}
			} // END - if
		} // END - foreach

		// Does this extension depends on an outstanding update of another update?
		if (!empty($EXT_UPDATE_DEPENDS)) {
			// Backup SQL commands and clear current
			$SQLs2 = $SQLs;
			$SQLs  = array();
			$test  = false;
			$ext_update = $EXT_UPDATE_DEPENDS;

			// Check for required file
			if (LOAD_EXTENSION($ext_update, "register", "", $dry_run, $SQLs)) {
				// If versions mismatch update extension first
				$ext_ver = GET_EXT_VERSION($ext_update);

				// Extension version set? If empty the extension is not registered
				if (empty($ext_ver)) {
					// Extension not registered so far so first load task's ID...
					$task = DETERMINE_EXTENSION_TASK_ID($ext_update);

					// Entry found?
					if ($task > 0) {
						// Try to register the extension
						$test = EXTENSION_REGISTER($ext_update, $task, $dry_run, false);
					} // END - if
				} elseif ($ext_ver != $EXT_VERSION) {
					// Ok, update this extension now
					EXTENSION_UPDATE($ext_update, $ext_ver, $dry_run);

					// All okay!
					$test = true;
				} else {
					// Nothing to register / update before...
					$test = true;
				}
			} else {
				// Required file for update does not exists!
				$test = true;
				// But this is fine for the first time...
			}

			// Finally restore previous SQLs
			$SQLs = $SQLs2; unset($SQLs2);
		} else {
			// Does not depend on an other extension
			$test = true;
		}

		// Switch back to register mode
		$EXT_LOAD_MODE = "register";

		// Remains true if extension registration reports no failtures
		$test = ($test && !$EXT_REPORTS_FAILURE);

		// Does everthing before wents ok?
		if ($test) {
			// "Dry-run-mode" activated?
			if (!$dry_run) {
				// Run installation pre-installation filters
				RUN_FILTER('pre_extension_installed', false);

				// Register extension
				$result = SQL_QUERY_ESC("INSERT INTO "._MYSQL_PREFIX."_extensions (ext_name, ext_active, ext_version) VALUES ('%s','%s','%s')",
					array($ext_name, $EXT_ALWAYS_ACTIVE, $EXT_VERSION), __FILE__, __LINE__);

				// Remove cache file(s) if extension is active
				RUN_FILTER('post_extension_installed', array('ext_name' => $ext_name, 'task_id' => $task_id));

				// In normal mode return a true on success
				$ret = true;
			} else {
				// Rewrite SQL command to keep { and } inside
				foreach ($SQLs as $key => $sql) {
					$sql = str_replace('{', "&#123;", str_replace('}', "&#125;", $sql));
					$SQLs[$key] = $sql;
				} // END - foreach

				// In  "dry-run" mode return array with all SQL commands
				$ret = $SQLs;

				// Remove all SQL commands
				unset($SQLs);
			}
		} else {
			// No, an error occurs while registering extension :-(
			$ret = false;
		}
	} elseif (($task_id > 0) && (!empty($ext_name))) {
		// Remove task from system when id and extension's name is valid
		$result = SQL_QUERY_ESC("DELETE LOW_PRIORITY FROM "._MYSQL_PREFIX."_task_system WHERE id=%s AND status='NEW' LIMIT 1",
			array(bigintval($task_id)), __FILE__, __LINE__);
	}

	// Is this the sql_patches?
	//* DEBUG: */ DEBUG_LOG(__FUNCTION__, __LINE__, ":{$ext_name}/{$EXT_LOAD_MODE}");
	if (($ext_name == "sql_patches") && (($EXT_LOAD_MODE == "register") || ($EXT_LOAD_MODE == "remove")) && (!$dry_run) && ($test)) {
		//* DEBUG: */ DEBUG_LOG(__FUNCTION__, __LINE__, ": LOAD!");
		if ($logout) {
			// Then redirect to logout
			LOAD_URL("modules.php?module=admin&logout=1&".$EXT_LOAD_MODE."=sql_patches");
		} else {
			// Add temporary filter
			REGISTER_FILTER('shutdown', 'REDIRECT_TO_LOGOUT_SQL_PATCHES', true, true);
			$GLOBALS['ext_load_mode'] = $EXT_LOAD_MODE;
		}
	} // END - if

	// Return status code
	return $ret;
}

// Run SQL queries for given extension id
// @TODO Change from ext_id to ext_name (not just even the variable! ;-) )
function EXTENSION_RUN_SQLS ($ext_id, $load_mode) {
	global $cacheInstance, $SQLs;

	// This shall never do a non-admin user!
	if (!IS_ADMIN()) return false;

	// Init array
	$SQLs = array();

	// By default no SQL has been executed
	$sqlRan = false;

	// Get extension's name
	$ext_name = GET_EXT_NAME($ext_id);
	if (empty($ext_name)) return false;

	// Load extension in detected mode
	//* DEBUG: */ DEBUG_LOG(__FUNCTION__, __LINE__, ":ext_name[{$ext_id}]={$ext_name}");
	LOAD_EXTENSION($ext_name, $load_mode, "", false, $SQLs);

	//* DEBUG: */ DEBUG_LOG(__FUNCTION__, __LINE__, ":SQLs::count=".count($SQLs)."");
	if ((is_array($SQLs) && (sizeof($SQLs) > 0))) {
		// Run SQL commands...
		RUN_FILTER('run_sqls');

		// Removal mode?
		if ($load_mode == "remove") {
			// Delete this extension (remember to remove it from your server *before* you click on welcome!
			$result = SQL_QUERY_ESC("DELETE LOW_PRIORITY FROM "._MYSQL_PREFIX."_extensions WHERE ext_name='%s' LIMIT 1",
				array($ext_name), __FILE__, __LINE__);
		} // END - if
	} // END - if

	// Remove cache file(s) if extension is active
	if (((EXT_IS_ACTIVE("cache")) || (GET_EXT_VERSION("cache") != "")) && (((SQL_AFFECTEDROWS() == 1)) || ($sqlRan === true) || ($load_mode == "activate") || ($load_mode == "deactivate"))) {
		// Run filters
		RUN_FILTER('post_extension_run_sql', $ext_name);
	} // END - if

	// Is this the sql_patches?
	//* DEBUG: */ DEBUG_LOG(__FUNCTION__, __LINE__, ": {$ext_id}/{$ext_name}/{$load_mode}");
	if (($ext_name == "sql_patches") && (($load_mode == "register") || ($load_mode == "remove"))) {
		// Then redirect to logout
		//* DEBUG: */ DEBUG_LOG(__FUNCTION__, __LINE__, ": LOAD!");
		LOAD_URL("modules.php?module=admin&logout=1&".$load_mode."=sql_patches");
	} // END - if
}

// Check if given extension is active
function EXT_IS_ACTIVE ($ext_name) {
	global $cacheArray, $_CONFIG;

	// Extensions are all inactive during installation
	if ((!isBooleanConstantAndTrue('mxchange_installed')) || (isBooleanConstantAndTrue('mxchange_installing')) || (empty($ext_name))) return false;

	// Not active is the default
	$active = "N";

	// Check cache
	if (isset($cacheArray['extensions']['ext_active'][$ext_name])) {
		// Load from cache
		//* DEBUG: */ DEBUG_LOG(__FUNCTION__, __LINE__, "CACHE! ext_name={$ext_name}");
		$active = $cacheArray['extensions']['ext_active'][$ext_name];

		// Count cache hits
		if (getConfig('cache_hits') > 0) { $_CONFIG['cache_hits']++; } else { $_CONFIG['cache_hits'] = 1; }
	} elseif (($ext_name == "cache") || (GET_EXT_VERSION("cache") == "")) {
		//* DEBUG: */ DEBUG_LOG(__FUNCTION__, __LINE__, "DB! ext_name={$ext_name}");
		// Load from database
		$result = SQL_QUERY_ESC("SELECT ext_active FROM "._MYSQL_PREFIX."_extensions WHERE ext_name='%s' LIMIT 1",
			array($ext_name), __FILE__, __LINE__);

		// Entry found?
		if (SQL_NUMROWS($result) == 1) {
			// Load entry
			list($active) = SQL_FETCHROW($result);
		} // END - if

		// Free result
		SQL_FREERESULT($result);

		// Write cache array
		//* DEBUG: */ echo $ext_name."[DB]: {$active}");
		$cacheArray['extensions']['ext_active'][$ext_name] = $active;
	} else {
		// Extension not active!
		//* DEBUG: */ echo $ext_name.": Not active!");
		$cacheArray['extensions']['ext_active'][$ext_name] = "N";
	}

	// Debug message
	//* DEBUG: */ DEBUG_LOG(__FUNCTION__, __LINE__, " ext_name={$ext_name},active={$active}");

	// Is this extension activated? (For admins we always have active extensions...)
	return ($active == "Y");
}
// Get version from extensions
function GET_EXT_VERSION ($ext_name) {
	global $cacheArray, $_CONFIG, $cacheInstance;
	$ext_ver = false;

	// Extensions are all inactive during installation
	if ((!isBooleanConstantAndTrue('mxchange_installed')) || (isBooleanConstantAndTrue('mxchange_installing'))) return "";
	//* DEBUG: */ DEBUG_LOG(__FUNCTION__, __LINE__, ": ext_name={$ext_name}");

	// Is the cache written?
	if (isset($cacheArray['extensions']['ext_version'][$ext_name])) {
		// Load data from cache
		//* DEBUG: */ DEBUG_LOG(__FUNCTION__, __LINE__, ": CACHE!");
		$ext_ver = $cacheArray['extensions']['ext_version'][$ext_name];

		// Count cache hits
		if (getConfig('cache_hits') > 0) $_CONFIG['cache_hits']++; else $_CONFIG['cache_hits'] = 1;
	} elseif (!is_object($cacheInstance)) {
		// Load from database
		$result = SQL_QUERY_ESC("SELECT ext_version FROM "._MYSQL_PREFIX."_extensions WHERE ext_name='%s' LIMIT 1",
			array($ext_name), __FILE__, __LINE__);

		// Is the extension there?
		if (SQL_NUMROWS($result) == 1) {
			// Load entry
			list($ext_ver) = SQL_FETCHROW($result);
		} // END - if

		// Free result
		SQL_FREERESULT($result);

		// Set cache
		$cacheArray['extensions']['ext_version'][$ext_name] = $ext_ver;
	}

	// Return result
	//* DEBUG: */ DEBUG_LOG(__FUNCTION__, __LINE__, ": ret={$ext_ver}");
	return $ext_ver;
}

// Updates a given extension with current extension version to latest version
function EXTENSION_UPDATE ($ext_name, $ext_ver, $dry_run = false) {
	// This shall never do a non-admin user!
	global $cacheInstance, $UPDATE_NOTES, $NOTES, $EXT_VER_HISTORY;
	global $EXT_UPDATE_DEPENDS, $EXT_VERSION, $INC_POOL, $cacheArray;

	// Init arrays
	$SQLs = array(); $INC_POOL = array();

	// Only admins are allowed to update extensions
	if ((!IS_ADMIN()) || (empty($ext_name))) return false;

	// Load extension in test mode
	LOAD_EXTENSION($ext_name, "test", $ext_ver, $dry_run, $SQLs);

	// Save version history
	$history = $EXT_VER_HISTORY;

	// Remove old SQLs array to prevent possible bugs
	$SQLs = array();

	// Check if version is updated
	if ((($EXT_VERSION != $ext_ver) || ($dry_run)) && (is_array($history))) {
		// Search for starting point
		$start = array_search($ext_ver, $history);
		$NOTES = "";

		// And load SQL queries in order of version history
		for ($idx = ($start + 1); $idx < sizeof($history); $idx++) {
			// Set extension version
			$cacheArray['update_ver'][$ext_name] = $history[$idx];

			// Load again...
			LOAD_EXTENSION($ext_name, "update", $cacheArray['update_ver'][$ext_name], $dry_run, $SQLs);

			if (!empty($EXT_UPDATE_DEPENDS)) {
				// Backup current SQL queries
				$cacheArray['update_sqls'][$ext_name] = $SQLs;

				// Is the extension there?
				if (GET_EXT_VERSION($EXT_UPDATE_DEPENDS) != "") {
					// Update another extension first!
					$test = EXTENSION_UPDATE($EXT_UPDATE_DEPENDS, GET_EXT_VERSION($EXT_UPDATE_DEPENDS), $dry_run);
				} else {
					// Register new extension
					$test = EXTENSION_REGISTER($EXT_UPDATE_DEPENDS, 0, $dry_run, false);
				}

				// Restore previous SQL queries
				$SQLs = $cacheArray['update_sqls'][$ext_name];
				unset($cacheArray['update_sqls'][$ext_name]);
			} // END - if

			// Add notes
			if (getConfig('verbose_sql') == "Y") {
				if (!empty($UPDATE_NOTES)) {
					// Update notes found
					$NOTES .= ("<div class=\"update_notes\"><strong>v".$cacheArray['update_ver'][$ext_name].":</strong><br />".$UPDATE_NOTES."</div>");
					$UPDATE_NOTES = "";
				} elseif ($cacheArray['update_ver'][$ext_name] == "0.0") {
					// Initial release
					$NOTES .= ("<div class=\"update_notes\"><strong>v".$cacheArray['update_ver'][$ext_name].":</strong><br />".INITIAL_RELEASE."</div>");
				} else {
					$NOTES .= ("<div class=\"update_notes\"><strong>v".$cacheArray['update_ver'][$ext_name].":</strong><br /><I>".NO_UPDATE_NOTES."</I></div>");
				}
			} // END - if
		} // END - for

		// In real-mode execute any existing includes
		if (!$dry_run) {
			$cacheArray['inc_pool'][$ext_name] = $INC_POOL;
			RUN_FILTER('load_includes');
			$INC_POOL = $cacheArray['inc_pool'][$ext_name];
			unset($cacheArray['inc_pool'][$ext_name]);
		} // END - if

		// Run SQLs
		RUN_FILTER('run_sqls', $dry_run);

		if (!$dry_run) {
			// Create task
			CREATE_EXTENSION_UPDATE_TASK(GET_CURRENT_ADMIN_ID(), $ext_name, $cacheArray['update_ver'][$ext_name], addslashes($NOTES));

			// Update extension's version
			$result = SQL_QUERY_ESC("UPDATE "._MYSQL_PREFIX."_extensions SET ext_version='%s' WHERE ext_name='%s' LIMIT 1",
				array($cacheArray['update_ver'][$ext_name], $ext_name), __FILE__, __LINE__);

			// Remove arrays
			unset($SQLs);
			unset($cacheArray['update_ver'][$ext_name]);

			// Run filters on success extension update
			RUN_FILTER('extension_update', $ext_name);
		} else {
			// In "dry-run" mode return array with SQL commands
			return $SQLs;
		}
	}
}

// Output verbose SQL table for extension
function EXTENSION_VERBOSE_TABLE ($queries = array(), $title = ADMIN_SQLS_EXECUTED_ON_REMOVAL, $dashed = "", $switch = false, $width = "100%") {
	global $SQLs;

	// Are there some queries in $queries?
	if (count($queries) > 0) {
		// Then use them instead!
		$SQLs = $queries;
	} // END - if

	// Init variables
	$S = false; $SW = 2; $i = 1;
	$OUT = "";

	// Do we have queries?
	if ((is_array($SQLs)) && (GET_EXT_VERSION("sql_patches") >= "0.0.7") && (getConfig('verbose_sql') == "Y")) {
		$OUT  = "<div align=\"center\">
<table border=\"0\" cellspacing=\"0\" cellpadding=\"0\" width=\"".$width."\" align=\"center\"".$dashed.">
<tr>
  <td colspan=\"2\" align=\"center\" class=\"admin_title bottom2\" height=\"24\">
    <strong>".$title.":</strong>
  </td>
</tr>\n";
		foreach ($SQLs as $idx => $sql) {
			$sql = trim($sql);
			if (!empty($sql)) {
				$S = true;
				$OUT .= "<tr>
  <td class=\"switch_sw".(3 - $SW)." bottom2\" style=\"padding-left: 2px\" width=\"30\">".$i.".</td>
  <td class=\"switch_sw".(3 - $SW)." bottom2\" style=\"padding-left: 5px; padding-right: 5px\">
    ".$sql."
  </td>
</tr>\n";
				if ($switch) $SW = 3 - $SW;
				$i++;
			}
		}
	}

	if ((!$S) && (GET_EXT_VERSION("sql_patches") >= "0.0.7") && (getConfig('verbose_sql') == "Y")) {
		// No addional SQL commands to run
		$OUT .= "<tr>
  <td colspan=\"2\" align=\"center\" class=\"switch_sw2 bottom2\" height=\"24\">
    <font class=\"admin_note\">".ADMIN_NO_ADDIONAL_SQLS."</font>
  </td>
</tr>\n";
	}

	if (!empty($OUT)) {
		// Add missing close-table tag
		$OUT .= "</table>
</div>\n";
	}

	// Return output
	return $OUT;
}

// Get extension name from id
function GET_EXT_NAME ($ext_id) {
	global $cacheArray, $_CONFIG;

	// Init extension name
	$ret = "";

	// Is cache there?
	if (isset($cacheArray['extensions']['ext_name'][$ext_id])) {
		// Load from cache
		$ret = $cacheArray['extensions']['ext_name'][$ext_id];

		// Count cache hits
		if (getConfig('cache_hits') > 0) { $_CONFIG['cache_hits']++; } else { $_CONFIG['cache_hits'] = 1; }
	} elseif (!EXT_IS_ACTIVE("cache")) {
		// Load from database
		$result = SQL_QUERY_ESC("SELECT ext_name FROM "._MYSQL_PREFIX."_extensions WHERE id=%s LIMIT 1",
			array(bigintval($ext_id)), __FILE__, __LINE__);
		list($ret) = SQL_FETCHROW($result);
		SQL_FREERESULT($result);
	}
	return $ret;
}

// Get extension id from name
function GET_EXT_ID ($ext_name) {
	global $cacheArray, $_CONFIG;

	// Init ID number
	$ret = 0;
	if (isset($cacheArray['extensions']['ext_id'][$ext_name])) {
		// Load from cache
		$ret = $cacheArray['extensions']['ext_id'][$ext_name];

		// Count cache hits
		if (getConfig('cache_hits') > 0) { $_CONFIG['cache_hits']++; } else { $_CONFIG['cache_hits'] = 1; }
	} elseif (!EXT_IS_ACTIVE("cache")) {
		// Load from database
		$result = SQL_QUERY_ESC("SELECT id FROM "._MYSQL_PREFIX."_extensions WHERE ext_name='%s' LIMIT 1",
			array($ext_name), __FILE__, __LINE__);
		list($ret) = SQL_FETCHROW($result);
		SQL_FREERESULT($result);
	}

	// Return value
	return $ret;
}

// Activate given extension
function ACTIVATE_EXTENSION ($ext_name) {
	// Activate the extension
	$result = SQL_QUERY_ESC("UPDATE "._MYSQL_PREFIX."_extensions SET ext_active='Y' WHERE ext_name='%s' LIMIT 1",
		array($ext_name), __FILE__, __LINE__);

	// Extension has been activated?
	if (SQL_AFFECTEDROWS() == 1) {
		// Then run all queries
		EXTENSION_RUN_SQLS(GET_EXT_ID($ext_name), "activate");
	} // END - if
}

// Deactivate given extension
function DEACTIVATE_EXTENSION($ext_name) {
	// Activate the extension
	$result = SQL_QUERY_ESC("UPDATE "._MYSQL_PREFIX."_extensions SET ext_active='N' WHERE ext_name='%s' LIMIT 1",
		array($ext_name), __FILE__, __LINE__);

	// Extension has been activated?
	if (SQL_AFFECTEDROWS() == 1) {
		// Then run all queries
		EXTENSION_RUN_SQLS(GET_EXT_ID($ext_name), "deactivate");

		// Create new task
		CREATE_EXTENSION_DEACTIVATION_TASK($ext_name);

		// Notify the admin
		SEND_ADMIN_NOTIFICATION(ADMIN_SUBJECT_EXTENSION_DEACTIVATED, "admin_ext_deactivated", array('ext_name' => $ext_name));
	} // END - if
}

// Checks wether the extension is older than given
function EXT_VERSION_IS_OLDER ($ext_name, $ext_ver) {
	// Get current extension version
	$currVersion = GET_EXT_VERSION($ext_name);

	// Remove all dots from both versions
	$currVersion = str_replace(".", "", $currVersion);
	$ext_ver = str_replace(".", "", $ext_ver);

	// Now compare both and return the result
	return ($currVersion < $ext_ver);
}

// Creates a new task for updated extension
function CREATE_EXTENSION_UPDATE_TASK ($admin_id, $ext_name, $ext_ver, $notes) {
	// Create subject line
	$subject = "[UPDATE-".$ext_name."-".$ext_ver.":] ".ADMIN_UPDATE_EXT_SUBJ;

	// Is the extension there?
	if (GET_EXT_VERSION($ext_name) != "") {
		// Check if task is not there
		if (DETERMINE_TASK_ID_BY_SUBJECT($subject) == 0) {
			// Task not created so it's a brand-new extension which we need to register and create a task for!
			SQL_QUERY_ESC("INSERT INTO "._MYSQL_PREFIX."_task_system (assigned_admin, userid, status, task_type, subject, text, task_created) VALUES ('%s','0','NEW','EXTENSION_UPDATE','%s','%s', UNIX_TIMESTAMP())",
				array($admin_id, $subject, $notes), __FILE__, __LINE__);
		} // END - if
	} // END - if
}

// Creates a new task for newly installed extension
function CREATE_NEW_EXTENSION_TASK ($admin_id, $subject, $ext) {
	// Not installed and do we have created a task for the admin?
	if ((DETERMINE_TASK_ID_BY_SUBJECT($subject) == 0) && (GET_EXT_VERSION($ext) == "")) {
		// Template file
		$tpl = sprintf("%stemplates/%s/html/ext/ext_%s.tpl",
			PATH,
			GET_LANGUAGE(),
			$ext
		);

		// Load text for task
		if (FILE_READABLE($tpl)) {
			// Load extension's own text template (HTML!)
			$msg = LOAD_TEMPLATE("ext_".$ext, true);
		} else {
			// Load default message
			$msg = LOAD_TEMPLATE("admin_new_ext", "", 0);
		}

		// Task not created so it's a brand-new extension which we need to register and create a task for!
		$result_insert = SQL_QUERY_ESC("INSERT INTO "._MYSQL_PREFIX."_task_system (assigned_admin, userid, status, task_type, subject, text, task_created)
VALUES (%s,0,'NEW','EXTENSION','%s','%s',UNIX_TIMESTAMP())",
			array(
				$admin_id,
				$subject,
				addslashes($msg),
			),  __FILE__, __LINE__, true, false
		);
	} // END - if
}

// Creates a task for automatically deactivated (deprecated) extension
function CREATE_EXTENSION_DEACTIVATION_TASK ($ext) {
	// Create subject line
	$subject = sprintf("[%s:] %s", $ext, TASK_SUBJ_EXTENSION_DEACTIVATED);

	// Not installed and do we have created a task for the admin?
	if ((DETERMINE_TASK_ID_BY_SUBJECT($subject) == 0) && (GET_EXT_VERSION($ext) != "")) {
		// Task not created so add it
		$result_insert = SQL_QUERY_ESC("INSERT INTO "._MYSQL_PREFIX."_task_system (assigned_admin, userid, status, task_type, subject, text, task_created)
VALUES (0,0,'NEW','EXTENSION_DEACTIVATION','%s','%s',UNIX_TIMESTAMP())",
			array(
				$subject,
				addslashes(LOAD_TEMPLATE("task_ext_deactivated", true, $ext)),
			),  __FILE__, __LINE__, true, false
		);
	} // END - if

	// Free memory
	SQL_FREERESULT($result);
}

// Checks if the module has a menu
function MODULE_HAS_MENU ($mod, $forceDb = false) {
	global $cacheArray, $_CONFIG;

	// All is false by default
	$ret = false;
	//* DEBUG: */ print __FUNCTION__."(<font color=\"#0000aa\">".__LINE__."</font>):mod={$mod},cache=".GET_EXT_VERSION("cache")."<br />\n";
	if (GET_EXT_VERSION("cache") >= "0.1.2") {
		// Cache version is okay, so let's check the cache!
		if (isset($cacheArray['modules']['has_menu'][$mod])) {
			// Check module cache and count hit
			$ret = ($cacheArray['modules']['has_menu'][$mod] == "Y");
			if (getConfig('cache_hits') > 0) { $_CONFIG['cache_hits']++; } else { $_CONFIG['cache_hits'] = 1; }
		} elseif (isset($cacheArray['extensions']['ext_menu'][$mod])) {
			// Check cache and count hit
			$ret = ($cacheArray['extensions']['ext_menu'][$mod] == "Y");
			if (getConfig('cache_hits') > 0) { $_CONFIG['cache_hits']++; } else { $_CONFIG['cache_hits'] = 1; }
		} elseif ((IS_ADMIN()) && ($mod == "admin")) {
			// Admin module has always a menu!
			$ret = true;
		}
	} elseif ((GET_EXT_VERSION("sql_patches") >= "0.3.6") && ((!EXT_IS_ACTIVE("cache")) || ($forceDb === true))) {
		// Check database for entry
		$result = SQL_QUERY_ESC("SELECT has_menu FROM "._MYSQL_PREFIX."_mod_reg WHERE module='%s' LIMIT 1",
			array($mod), __FILE__, __LINE__);

		// Entry found?
		if (SQL_NUMROWS($result) == 1) {
			// Load "has_menu" column
			list($has_menu) = SQL_FETCHROW($result);

			// Fake cache... ;-)
			$cacheArray['extensions']['ext_menu'][$mod] = $has_menu;

			// Does it have a menu?
			$ret = ($has_menu == "Y");
		} // END  - if

		// Free memory
		SQL_FREERESULT($result);
	} elseif (GET_EXT_VERSION("sql_patches") == "") {
		// No sql_patches installed, so maybe in admin area?
		$ret = ((IS_ADMIN()) && ($mod == "admin")); // Then there is a menu!
	}

	// Return status
	//* DEBUG: */ var_dump($ret);
	return $ret;
}

// Determines the task id for given extension
function DETERMINE_EXTENSION_TASK_ID ($ext_name) {
	// Default is not found
	$task_id = 0;

	// Search for extension task's id
	$result = SQL_QUERY_ESC("SELECT id FROM "._MYSQL_PREFIX."_task_system WHERE task_type='EXTENSION' AND subject='[%s:]' LIMIT 1",
		array($ext_name), __FILE__, __LINE__);

	// Entry found?
	if (SQL_NUMROWS($result) == 1) {
		// Task found so load task's ID and register extension...
		list($task_id) = SQL_FETCHROW($result);
	} // END - if

	// Free result
	SQL_FREERESULT($result);

	// Return it
	return $task_id;
}

// Determines the task id for given subject
function DETERMINE_TASK_ID_BY_SUBJECT ($subject) {
	// Default is not found
	$task_id = 0;

	// Search for task id
	$result = SQL_QUERY_ESC("SELECT id FROM "._MYSQL_PREFIX."_task_system WHERE subject LIKE '%s%%' LIMIT 1",
		array($subject), __FILE__, __LINE__);

	// Entry found?
	if (SQL_NUMROWS($result) == 1) {
		// Task found so load task's ID and register extension...
		list($task_id) = SQL_FETCHROW($result);
	} // END - if

	// Free result
	SQL_FREERESULT($result);

	// Return it
	return $task_id;
}

//
?>
